//--
//--	animwind.cpp
//--
//--  This is the animation frame window.  It offers the basic window services
//--  to the game.
//
//    Handles the Client Device context.
//
//    Handles the main off screen buffer.  This is the one that will be used
//    for Windows paint events.
//      -Creates an identity palette in it's constructor
//      -SetupWindow calls the virtual function InitPalette to
//       allow the derived to create the palette it needs for it's bitmaps
//      -Creates the buffer using the client size and the palette
//
//    After the buffer is created, it is cleared.  You need to set backgroundROP
//    in your CONSTRUCTOR to the Raster Op Code you want.  
//
//    In addition to Paint(), has a function that Blits a rectangle
//       from the buffer to the screen.  Allowing you to bypass the
//       messaging system in order to invalidate a rectangle.
//
//    The client area will be the size represented by CLIENT_SCREEN_WIDTH and
//    CLIENT_SCREEN_HEIGHT.  The window is placed at Point(10,10).  There were
//    problems when it was placed at (0,0).  I think this was due to the
//    fact that I am not using a sizable frame.


#include <owl\window.h>
#include <owl\framewin.h>
#include <owl\module.h>
#include <owl\point.h>
#include <owl\dc.h>
#include <owl\gdiobjec.h>
#pragma hdrstop
#include "palette.h"
#include "wingdc.h"
#include "animwind.h"
#include "buffer.h"


//--------------------------------------------------
//  TBitmapAnimatorWindow::TBitmapAnimatorWindow()
//--------------------------------------------------
TBitmapAnimatorWindow::TBitmapAnimatorWindow(TWindow *pParWin,
											 const char far *pszTitle) :
									 TFrameWindow(pParWin,pszTitle)
{

	buffer = 0; // Allocated in SetupWindow()
	pClientDC = 0;  // Allocated in SetupWindow()
	pPalette = new TIdentityPalette(); // Initialize you palette
	backgroundROP = BLACKNESS;

}


//---------------------------------------------------
//  TBitmapAnimatorWindow::~TBitmapAnimatorWindow()
//---------------------------------------------------
TBitmapAnimatorWindow::~TBitmapAnimatorWindow()
{

	if (pClientDC) delete pClientDC;
	delete pPalette;
	delete buffer;

	//Destroy();
}

void TBitmapAnimatorWindow::CleanupWindow(){

	pClientDC->RestorePalette(); // For some reason this line needs to be here
	// rather than in the destructor.  If it is in the destructor,it causes a
	// leak under win3.1   Also under win3.1, if the line is here, it caused
	// an error when the window is closed while animation is in progress.
	// Under Win95, I don't get the error.

	TWindow::CleanupWindow();
}

//-------------------------------------------
//  TBitmapAnimatorWindow::GetWindowClass()
//-------------------------------------------
void TBitmapAnimatorWindow::GetWindowClass(WNDCLASS &WndClass)
{
	TFrameWindow::GetWindowClass(WndClass);

	WndClass.style = CS_OWNDC;  // ???
	WndClass.hbrBackground = (HBRUSH)GetStockObject(NULL_BRUSH);// ???
}


//----------------------------------------
//  TBitmapAnimatorWindow::SetupWindow()
//----------------------------------------
void TBitmapAnimatorWindow::SetupWindow()
{
	// Pass to base class
	TFrameWindow::SetupWindow();

	// Let derived classes fill the identity palette
	InitPalette();

	// Setup our owned DC
	pClientDC = new TClientDC(HWindow);
	pClientDC->SetStretchBltMode(COLORONCOLOR); //???
	pClientDC->SelectObject(*pPalette);
	pClientDC->RealizePalette();


	// Size the client area

	TRect wRect = GetWindowRect();
	TRect cRect = GetClientRect();// width == right, height == bottom

	// The size of the frame window is the size of the client area
	// plus the amount needed for the caption, menu bar, frame and ???
	MoveWindow(10,10,CLIENT_SCREEN_WIDTH + (wRect.Width() - cRect.right),
						CLIENT_SCREEN_HEIGHT + wRect.Height() - cRect.bottom);

	// Make the buffer the size of the client area.
	buffer = new TBuffer(TSize(CLIENT_SCREEN_WIDTH, CLIENT_SCREEN_HEIGHT),
								pPalette);
	buffer->EraseBuffer(backgroundROP);

	clientRect = GetClientRect();  // load this for derived classes use
}

//----------------------------------
//  TBitmapAnimatorWindow::Paint()
// Blit the dirty rect portion of the buffer to the screen.
//----------------------------------
void TBitmapAnimatorWindow::Paint(TDC &dc, bool, TRect &UpdateRect)
{

	dc.SelectObject(*pPalette);
	dc.RealizePalette();

	if (buffer->pBufferDC && !UpdateRect.IsEmpty())
	{
		buffer->pBufferDC->WinGBitBlt((HDC)*pClientDC,
								UpdateRect.left,
								UpdateRect.top,
								UpdateRect.Width(),
								UpdateRect.Height(),
								UpdateRect.left,
								UpdateRect.top);
	}
}
void TBitmapAnimatorWindow::UpdateScreen(TRect &UpdateRect)
{

	if (buffer->pBufferDC && !UpdateRect.IsEmpty())
	{
		buffer->pBufferDC->WinGBitBlt((HDC)*pClientDC,
								UpdateRect.left,
								UpdateRect.top,
								UpdateRect.Width(),
								UpdateRect.Height(),
								UpdateRect.left,
								UpdateRect.top);
	}
}



//---------------------------------------------
//  TBitmapAnimatorWindow::EvPaletteChanged()
//---------------------------------------------
void TBitmapAnimatorWindow::EvPaletteChanged(HWND hwnd)
{
	if (hwnd != HWindow)  EvQueryNewPalette();
}


//----------------------------------------------
//  TBitmapAnimatorWindow::EvQueryNewPalette()
//----------------------------------------------
bool TBitmapAnimatorWindow::EvQueryNewPalette()
{
	pClientDC->SelectObject(*pPalette);
	int nClrChanged = pClientDC->RealizePalette();
	if (nClrChanged)  Invalidate();
	return (nClrChanged != 0);

}



