//--
//--	buffer.cpp
//-- This class encapsulates an off screen buffer.  It uses a WinGDC.
//-- Construct it with the buffer size you want and a palette.
//--  Works with 8 bit color.

//-- After creating a buffer you need to initialize it.
//-- You should call EraseBuffer with
//-- either BLACKNESS or WHITENESS, or draw your own background to the buffer
//--
//-- This class allows you to create multiple buffers.
//
//--	TBuffer Class Module

#include <owl\window.h>
#include <owl\framewin.h>
#include <owl\module.h>
#include <owl\point.h>
#include <owl\dc.h>
#include <owl\gdiobjec.h>
#pragma hdrstop
#include "wingdc.h"
#include "palette.h"
#include "buffer.h"

//--------------------------------------------------
//  TBuffer::TBuffer()
//--------------------------------------------------
TBuffer::TBuffer(TSize size, TIdentityPalette *iPalette)
{

	// Init WinG stuff
	// Allocate a TWinGDC
	pPalette = iPalette;
	try
	{
		pBufferDC = new TWinGDC();
	}
	catch (...)
	{
		pBufferDC = 0;
		::PostQuitMessage(1);
		return;
	}
	HBITMAP hBM = BuildOffScreenDIB(size.cx,size.cy);
	nWidth = size.cx;
	nHeight = size.cy;
	hbmOldMonoBitmap = (HBITMAP)::SelectObject((HDC)*pBufferDC,hBM);
}


//---------------------------------------------------
//  TBuffer::~TBuffer()
//---------------------------------------------------
TBuffer::~TBuffer()
{
	if (pBufferDC)
	{
		HBITMAP hBM = (HBITMAP)::SelectObject((HDC)*pBufferDC,
															hbmOldMonoBitmap);
		::DeleteObject(hBM);
	}
	delete pBufferDC;
	pBufferDC = 0;
}


//----------------------------------------------
//  TBuffer::BuildOffScreenDIB()
//----------------------------------------------
HBITMAP TBuffer::BuildOffScreenDIB(int nXSize, int nYSize)
{
	if (!pBufferDC)  return 0;

	// Query WinG for fastest DIB format - force to 8-bit/no compression
	if (pBufferDC->WinGRecommendDIBFormat((BITMAPINFO far *)&BufferHeader))
	{
		BufferHeader.Header.biBitCount = 8;
		BufferHeader.Header.biCompression = BI_RGB;
		lBMPOrientation = BufferHeader.Header.biHeight > 0 ? 1 : -1;
	}
	// If WinGRecommendDIBFormat() failed we'll just build our own 8-bit DIB
	else
	{
		BufferHeader.Header.biSize = sizeof(BITMAPINFOHEADER);
		BufferHeader.Header.biPlanes = 1;
		BufferHeader.Header.biBitCount = 8;
		BufferHeader.Header.biCompression = BI_RGB;
		BufferHeader.Header.biSizeImage = 0;
		BufferHeader.Header.biClrUsed = 0;
		BufferHeader.Header.biClrImportant = 0;
	}

	BufferHeader.Header.biWidth = nXSize;
	BufferHeader.Header.biHeight = nYSize * lBMPOrientation;

	// Set up the offscreen DIBs color table from the colors in pPalette
	//   Allocate and init a temporary PALETTEENTRY array
	uint nNumEntries = pPalette->GetNumEntries();
	PALETTEENTRY *pPalEntries = new PALETTEENTRY[nNumEntries];
	//   Fill the array with the palette entries
	::GetPaletteEntries((HPALETTE)*pPalette,0,nNumEntries,pPalEntries);
	//   Copy identity palette entries to our off-screen DIB header
	//   so that there will be an exact match between them and the sys palette
	for (uint nCounter = 0; nCounter < nNumEntries; ++nCounter)
	{
		BufferHeader.aColors[nCounter].rgbRed =
									  pPalEntries[nCounter].peRed;
		BufferHeader.aColors[nCounter].rgbGreen =
									  pPalEntries[nCounter].peGreen;
		BufferHeader.aColors[nCounter].rgbBlue =
									  pPalEntries[nCounter].peBlue;
		BufferHeader.aColors[nCounter].rgbReserved = 0;
	}
	//   Free the temporary PALETTEENTRY array
	delete pPalEntries;

	return (pBufferDC->WinGCreateBitmap((BITMAPINFO far *)&BufferHeader.Header,
															&pOffScreenBits));
}


//------------------------------------------------
//  TBuffer::RebuildOffScreenDIB()
//------------------------------------------------

void TBuffer::RebuildOffScreenDIB(int nXSize, int nYSize)
{
	if(pBufferDC)
	{
		HBITMAP hBM;

		if (!nXSize)  nXSize = 32;   // defaults
		if (!nYSize)  nYSize = 32;   // defaults

		BufferHeader.Header.biWidth = nXSize;
		BufferHeader.Header.biHeight = nYSize * lBMPOrientation;
		hBM = pBufferDC->WinGCreateBitmap((BITMAPINFO far *)
												&BufferHeader.Header,
															&pOffScreenBits);

		//  Select it in and delete the old one
		hBM = (HBITMAP)::SelectObject((HDC)*pBufferDC,hBM);
		::DeleteObject(hBM);
		EraseBuffer(BLACKNESS);
	}
}

void TBuffer::EraseBuffer(uint32 rop)
{
	pBufferDC->PatBlt(0,0,(int)BufferHeader.Header.biWidth,
							BufferHeader.Header.biHeight < 0 ?
									(int)BufferHeader.Header.biHeight * -1:
									(int)BufferHeader.Header.biHeight,
							rop);
}

